/**
 * This class parses and represents a single read in a SAM/BAM file.
 *
 * This is used instead of parsing directly via jBinary in order to:
 * - Make the parsing lazy (for significant performance wins)
 * - Make the resulting object more precisely typed.
 *
 * Parsing reads using SamRead is ~2-3x faster than using jBinary and
 * ThinAlignment directly.
 *
 * 
 */
'use strict';



var _Alignment = require("../Alignment");

var _jdataview = _interopRequireDefault(require("jdataview"));
var _jbinary = _interopRequireDefault(require("jbinary"));
var _underscore = _interopRequireDefault(require("underscore"));
var _bamTypes = _interopRequireDefault(require("./formats/bamTypes"));
var _ContigInterval = _interopRequireDefault(require("../ContigInterval"));

var _pileuputils = require("../viz/pileuputils");function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError("Cannot call a class as a function");}}function _defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ("value" in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}function _createClass(Constructor, protoProps, staticProps) {if (protoProps) _defineProperties(Constructor.prototype, protoProps);if (staticProps) _defineProperties(Constructor, staticProps);return Constructor;}

// TODO: Make more extensive use of the jBinary specs.


var CIGAR_OPS = ['M', 'I', 'D', 'N', 'S', 'H', 'P', '=', 'X'];

var SEQUENCE_VALUES = ['=', 'A', 'C', 'M', 'G', 'R', 'S', 'V',
'T', 'W', 'Y', 'H', 'K', 'D', 'B', 'N'];


function strandFlagToString(reverseStrand) {
  return reverseStrand ? '-' : '+';
}var


SamRead /* implements Alignment */ = /*#__PURE__*/function () {










  // cached values




  /**
   * @param buffer contains the raw bytes of the serialized BAM read. It must
   *     contain at least one full read (but may contain more).
   * @param offset records where this alignment is located in the BAM file. It's
   *     useful as a unique ID for alignments.
   * @param ref is the human-readable name of the reference/contig (the binary
   *     encoding only contains an ID).
   */
  function SamRead(buffer, offset, ref) {_classCallCheck(this, SamRead);
    this.buffer = buffer;
    this.offset = offset;

    // Go ahead and parse a few fields immediately.
    var jv = this._getJDataView();
    this.refID = jv.getInt32(0);
    this.ref = ref;
    this.pos = jv.getInt32(4);
    this.l_seq = jv.getInt32(16);
    this.cigarOps = this._getCigarOps();
    this.name = this._getName();
  }_createClass(SamRead, [{ key: "toString", value:

    function toString() {
      var stop = this.pos + this.l_seq;
      return "".concat(this.ref, ":").concat(1 + this.pos, "-").concat(stop);
    } }, { key: "_getJDataView", value:

    function _getJDataView() {
      var b = this.buffer;
      return new _jdataview["default"](b, 0, b.byteLength, true /* little endian */);
    }

    /**
     * Returns an identifier which is unique within the BAM file.
     */ }, { key: "getKey", value:
    function getKey() {
      return this.offset.toString();
    } }, { key: "_getName", value:

    function _getName() {
      var jv = this._getJDataView();
      var l_read_name = jv.getUint8(8);
      jv.seek(32); // the read-name starts at byte 32
      return jv.getString(l_read_name - 1); // ignore null-terminator
    } }, { key: "getFlag", value:

    function getFlag() {
      return this._getJDataView().getUint16(14);
    } }, { key: "getInferredInsertSize", value:

    function getInferredInsertSize() {
      return this._getJDataView().getUint16(28);
    } }, { key: "getStrand", value:

    function getStrand() {
      return strandFlagToString(this.getFlag() & _bamTypes["default"].Flags.READ_STRAND);
    }

    // TODO: get rid of this; move all methods into SamRead.
  }, { key: "getFull", value: function getFull() {
      if (this._full) return this._full;
      var jb = new _jbinary["default"](this.buffer, _bamTypes["default"].TYPE_SET);
      var full = jb.read(_bamTypes["default"].ThickAlignment, 0);
      this._full = full;
      return full;
    } }, { key: "getInterval", value:

    function getInterval() {
      if (this._interval) return this._interval; // use the cache
      var interval = new _ContigInterval["default"](this.ref,
      this.pos,
      this.pos + this.getReferenceLength() - 1);
      return interval;
    } }, { key: "intersects", value:

    function intersects(interval) {
      return interval.intersects(this.getInterval());
    } }, { key: "_getCigarOps", value:

    function _getCigarOps() {
      var jv = this._getJDataView(),
      l_read_name = jv.getUint8(8),
      n_cigar_op = jv.getUint16(12),
      pos = 32 + l_read_name,
      cigar_ops = new Array(n_cigar_op);
      for (var i = 0; i < n_cigar_op; i++) {
        var v = jv.getUint32(pos + 4 * i);
        cigar_ops[i] = {
          op: CIGAR_OPS[v & 0xf],
          length: v >> 4 };

      }
      return cigar_ops;
    }

    /**
     * Returns per-base quality scores from 0-255.
     */ }, { key: "getQualityScores", value:
    function getQualityScores() {
      var jv = this._getJDataView(),
      l_read_name = jv.getUint8(8),
      n_cigar_op = jv.getUint16(12),
      l_seq = jv.getInt32(16),
      pos = 32 + l_read_name + 4 * n_cigar_op + Math.ceil(l_seq / 2);
      return jv.getBytes(l_seq, pos, true /* little endian */, true /* toArray */);
    } }, { key: "getCigarString", value:

    function getCigarString() {
      return (0, _Alignment.makeCigarString)(this.cigarOps);
    } }, { key: "getQualPhred", value:

    function getQualPhred() {
      return makeAsciiPhred(this.getQualityScores());
    } }, { key: "getSequence", value:

    function getSequence() {
      if (this._seq) return this._seq;
      var jv = this._getJDataView(),
      l_read_name = jv.getUint8(8),
      n_cigar_op = jv.getUint16(12),
      l_seq = jv.getInt32(16),
      pos = 32 + l_read_name + 4 * n_cigar_op,
      basePairs = new Array(l_seq),
      numBytes = Math.ceil(l_seq / 2);

      for (var i = 0; i < numBytes; i++) {
        var b = jv.getUint8(pos + i);
        basePairs[2 * i] = SEQUENCE_VALUES[b >> 4];
        if (2 * i + 1 < l_seq) {
          basePairs[2 * i + 1] = SEQUENCE_VALUES[b & 0xf];
        }
      }

      var seq = basePairs.join('');
      this._seq = seq;
      return seq;
    }

    // Returns the length of the alignment from first aligned read to last aligned read.
  }, { key: "getReferenceLength", value: function getReferenceLength() {
      return SamRead.referenceLengthFromOps(this.cigarOps);
    } }, { key: "getCoverage", value:

    function getCoverage(referenceSource) {
      return {
        range: this.getInterval(),
        opInfo: (0, _pileuputils.getOpInfo)(this, referenceSource) };

    } }, { key: "getMateProperties", value:

    function getMateProperties() {
      var jv = this._getJDataView(),
      flag = jv.getUint16(14);
      if (!(flag & _bamTypes["default"].Flags.READ_PAIRED)) return null;

      var nextRefId = jv.getInt32(20),
      nextPos = jv.getInt32(24),
      nextStrand = strandFlagToString(flag & _bamTypes["default"].Flags.MATE_STRAND);

      return {
        // If the mate is on another contig, there's no easy way to get its string name.
        ref: nextRefId == this.refID ? this.ref : null,
        pos: nextPos,
        strand: nextStrand };

    } }, { key: "debugString", value:

    function debugString() {
      var f = this.getFull();

      return "Name: ".concat(this.name, "\nFLAG: ").concat(
      this.getFlag(), "\nPosition: ").concat(
      this.getInterval().toString(), "\nCIGAR: ").concat(
      this.getCigarString(), "\nSequence: ").concat(
      f.seq, "\nQuality:  ").concat(
      this.getQualPhred(), "\nTags: ").concat(
      JSON.stringify(f.auxiliary, null, '  '), "\n    ");

    } }], [{ key: "referenceLengthFromOps", value:

    function referenceLengthFromOps(ops) {
      var refLength = 0;
      ops.forEach(function (_ref) {var op = _ref.op,length = _ref.length;
        switch (op) {
          case 'M':
          case 'D':
          case 'N':
          case '=':
          case 'X':
            refLength += length;}

      });
      return refLength;
    } }]);return SamRead;}();


// Convert an array of Phred scores to a printable string.
function makeAsciiPhred(qualities) {
  if (qualities.length === 0) return '';
  if (_underscore["default"].every(qualities, function (x) {return x == 255;})) return '*';
  return qualities.map(function (q) {return String.fromCharCode(33 + q);}).join('');
}

module.exports = SamRead;