/**
 * This exposes the main entry point into pileup.js.
 * 
 */
'use strict';


var _types = require("./types");



var _underscore = _interopRequireDefault(require("underscore"));
var _react = _interopRequireDefault(require("react"));
var _reactDom = _interopRequireDefault(require("react-dom"));
var _htmlToImage = _interopRequireDefault(require("html-to-image"));
var _fileSaver = _interopRequireDefault(require("file-saver"));


var _TwoBitDataSource = _interopRequireDefault(require("./sources/TwoBitDataSource"));
var _CytoBandDataSource = _interopRequireDefault(require("./sources/CytoBandDataSource"));

var _BigBedDataSource = _interopRequireDefault(require("./sources/BigBedDataSource"));
var _VcfDataSource = _interopRequireDefault(require("./sources/VcfDataSource"));
var _BamDataSource = _interopRequireDefault(require("./sources/BamDataSource"));
var _EmptySource = _interopRequireDefault(require("./sources/EmptySource"));

var _utils = _interopRequireDefault(require("./utils"));
var _Interval = _interopRequireDefault(require("./Interval"));


var _GA4GHAlignmentJson = _interopRequireDefault(require("./json/GA4GHAlignmentJson"));
var _GA4GHVariantJson = _interopRequireDefault(require("./json/GA4GHVariantJson"));
var _GA4GHFeatureJson = _interopRequireDefault(require("./json/GA4GHFeatureJson"));
var _IdiogramJson = _interopRequireDefault(require("./json/IdiogramJson"));


var _GA4GHAlignmentSource = _interopRequireDefault(require("./sources/GA4GHAlignmentSource"));
var _GA4GHVariantSource = _interopRequireDefault(require("./sources/GA4GHVariantSource"));
var _GA4GHFeatureSource = _interopRequireDefault(require("./sources/GA4GHFeatureSource"));
var _GA4GHGeneSource = _interopRequireDefault(require("./sources/GA4GHGeneSource"));


var _CoverageTrack = _interopRequireDefault(require("./viz/CoverageTrack"));
var _GenomeTrack = _interopRequireDefault(require("./viz/GenomeTrack"));
var _GeneTrack = _interopRequireDefault(require("./viz/GeneTrack"));
var _FeatureTrack = _interopRequireDefault(require("./viz/FeatureTrack"));
var _IdiogramTrack = _interopRequireDefault(require("./viz/IdiogramTrack"));
var _LocationTrack = _interopRequireDefault(require("./viz/LocationTrack"));
var _PileupTrack = _interopRequireDefault(require("./viz/PileupTrack"));
var _ScaleTrack = _interopRequireDefault(require("./viz/ScaleTrack"));
var _VariantTrack = _interopRequireDefault(require("./viz/VariantTrack"));
var _GenotypeTrack = _interopRequireDefault(require("./viz/GenotypeTrack"));
var _Root = _interopRequireDefault(require("./Root"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };} // Data sources
// Data sources from json
// GA4GH sources
// Visualizations






















function findReference(tracks) {
  return _underscore["default"].find(tracks, function (t) {return !!t.track.isReference;});
}


function create(elOrId, params) {
  var ref = /*#__PURE__*/_react["default"].createRef();

  var el = typeof elOrId == 'string' ? document.getElementById(elOrId) : elOrId;
  if (!el) {
    throw new Error("Attempted to create pileup with non-existent element ".concat(elOrId.toString()));
  }

  var vizTracks = params.tracks.map(function (track) {
    var source = track.data ? track.data : track.viz.component.defaultSource;
    if (!source) {
      var displayName = track.viz.component.displayName != null ? track.viz.component.displayName : 'track';
      throw new Error(
      "Track '".concat(displayName, "' doesn't have a default ") + "data source; you must specify one when initializing it.");


    }

    return { visualization: track.viz, source: source, track: track };
  });

  var referenceTrack = findReference(vizTracks);
  if (!referenceTrack) {
    throw new Error('You must include at least one track with type=reference');
  }

  _reactDom["default"].render( /*#__PURE__*/_react["default"].createElement(_Root["default"], { referenceSource: referenceTrack.source,
    tracks: vizTracks,
    initialRange: params.range,
    ref: ref }), el);

  //if the element doesn't belong to document DOM observe DOM to detect
  //when it's attached
  var observer = null;
  var body = document.body;
  if (!body) throw new Error("Failed to match: document.body");

  if (!body.contains(el)) {
    observer = new MutationObserver(function (mutations) {
      mutations.forEach(function (mutation) {
        if (mutation.type === 'childList') {
          var added = false;
          for (var i = 0; i < mutation.addedNodes.length; i++) {
            //when added element is element where we visualize pileup
            //or it contains element where we visualize pileup
            //then we will have to update component
            if (mutation.addedNodes[i] === el || mutation.addedNodes[i].contains(el)) {
              added = true;
            }
          }
          if (added) {
            if (ref.current) {
              ref.current.setState({ updateSize: true });
            } else {
              throw 'ReactElement was not initialized properly';
            }
          }
        }
      });
    });
    // configuration of the observer:
    var config = { attributes: true, childList: true, characterData: true, subtree: true };

    // start observing document
    observer.observe(document, config);
  }

  return {
    setRange: function setRange(range) {
      if (ref.current === null) {
        throw 'Cannot call setRange on a destroyed pileup';
      }
      ref.current.handleRangeChange(range);
    },
    getRange: function getRange() {
      if (ref.current === null) {
        throw 'Cannot call getRange on a destroyed pileup';
      }
      if (ref.current.state.range != null) {
        return _underscore["default"].clone(ref.current.state.range);
      } else {
        throw 'Cannot call getRange on non-existent range';
      }
    },
    zoomIn: function zoomIn() {
      if (ref.current === null) {
        throw 'Cannot call zoomIn on a destroyed pileup';
      }
      var r = this.getRange();
      var iv = _utils["default"].scaleRange(new _Interval["default"](r.start, r.stop), _utils["default"].ZOOM_FACTOR.IN);
      var newRange = {
        contig: r.contig,
        start: iv.start,
        stop: iv.stop };

      this.setRange(newRange);
      return newRange;
    },
    zoomOut: function zoomOut() {
      if (ref.current === null) {
        throw 'Cannot call zoomOut on a destroyed pileup';
      }
      var r = this.getRange();
      var iv = _utils["default"].scaleRange(new _Interval["default"](r.start, r.stop), _utils["default"].ZOOM_FACTOR.OUT);
      var newRange = {
        contig: r.contig,
        start: iv.start,
        stop: iv.stop };

      this.setRange(newRange);
      return newRange;
    },
    toSVG: function toSVG(filepath) {
      if (ref.current === null) {
        throw 'Cannot call toSVG on a destroyed pileup';
      }

      return _htmlToImage["default"].toSvgDataURL(el).
      then(function (svg) {
        if (filepath != null) {
          window.saveAs(svg, filepath);
        }
        return svg;
      });
    },
    destroy: function destroy() {
      if (!vizTracks) {
        throw 'Cannot call destroy() twice on the same pileup';
      }
      vizTracks.forEach(function (_ref) {var source = _ref.source;
        source.off();
      });
      _reactDom["default"].unmountComponentAtNode(el);
      ref.current = null;
      referenceTrack = null;
      vizTracks = null;

      // disconnect observer if it was created
      if (observer !== null && observer !== undefined) {
        observer.disconnect();
      }
    } };

}



function makeVizObject(component) {
  return function (options) {
    // $FlowIgnore: TODO remove flow suppression
    options = _underscore["default"].extend({}, component.defaultOptions, options);
    return { component: component, options: options };
  };
}

var pileup = {
  create: create,
  formats: {
    bam: _BamDataSource["default"].create,
    alignmentJson: _GA4GHAlignmentJson["default"].create,
    variantJson: _GA4GHVariantJson["default"].create,
    featureJson: _GA4GHFeatureJson["default"].create,
    idiogramJson: _IdiogramJson["default"].create,
    cytoBand: _CytoBandDataSource["default"].create,
    vcf: _VcfDataSource["default"].create,
    twoBit: _TwoBitDataSource["default"].create,
    bigBed: _BigBedDataSource["default"].create,
    GAReadAlignment: _GA4GHAlignmentSource["default"].create,
    GAVariant: _GA4GHVariantSource["default"].create,
    GAFeature: _GA4GHFeatureSource["default"].create,
    GAGene: _GA4GHGeneSource["default"].create,
    empty: _EmptySource["default"].create },

  viz: {
    coverage: makeVizObject(_CoverageTrack["default"]),
    genome: makeVizObject(_GenomeTrack["default"]),
    genes: makeVizObject(_GeneTrack["default"]),
    features: makeVizObject(_FeatureTrack["default"]),
    idiogram: makeVizObject(_IdiogramTrack["default"]),
    location: makeVizObject(_LocationTrack["default"]),
    scale: makeVizObject(_ScaleTrack["default"]),
    variants: makeVizObject(_VariantTrack["default"]),
    genotypes: makeVizObject(_GenotypeTrack["default"]),
    pileup: makeVizObject(_PileupTrack["default"]) },

  "enum": {
    variants: {
      allelFrequencyStrategy: _types.AllelFrequencyStrategy } },


  version: '0.7.0' };


module.exports = pileup;

// Export a global until the distributed package works with CommonJS
// See https://github.com/hammerlab/pileup.js/issues/136
if (typeof window !== 'undefined') {
  window.pileup = pileup;
}