/**
 * A data source which reads genes from a GA4GH Feature protocol.
 * 
 */
'use strict';

var _underscore = _interopRequireDefault(require("underscore"));
var _backbone = require("backbone");

var _ContigInterval = _interopRequireDefault(require("../ContigInterval"));



var _gene = _interopRequireDefault(require("../data/gene"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}

var BASE_PAIRS_PER_FETCH = 100;
var GENES_PER_REQUEST = 400;
var ZERO_BASED = false;


function create(spec) {
  var url = spec.endpoint + '/features/search';

  var genes = {};

  // Ranges for which we have complete information -- no need to hit network.
  var coveredRanges = [];

  function addFeaturesFromResponse(response) {
    if (response.features === undefined) {
      return;
    }

    response.features.forEach(function (ga4ghFeature) {
      var contigInterval = new _ContigInterval["default"](ga4ghFeature.referenceName, ga4ghFeature.start, ga4ghFeature.end);

      var key = ga4ghFeature.id + contigInterval.toString();
      if (key in genes) return;
      var gene = _gene["default"].fromGA4GH(ga4ghFeature);
      genes[key] = gene;
    });
  }

  function rangeChanged(newRange) {
    var interval = new _ContigInterval["default"](newRange.contig, newRange.start, newRange.stop);
    if (interval.isCoveredBy(coveredRanges)) return;

    interval = interval.round(BASE_PAIRS_PER_FETCH, ZERO_BASED);

    // select only intervals not yet loaded into coveredRangesß
    var intervals = interval.complementIntervals(coveredRanges);

    // We "cover" the interval immediately (before the reads have arrived) to
    // prevent duplicate network requests.
    coveredRanges.push(interval);
    coveredRanges = _ContigInterval["default"].coalesce(coveredRanges);

    intervals.forEach(function (i) {
      fetchFeaturesForInterval(i, null, 1);
    });
  }

  function notifyFailure(message) {
    o.trigger('networkfailure', message);
    o.trigger('networkdone');
    console.warn(message);
  }

  function fetchFeaturesForInterval(range,
  pageToken,
  numRequests) {
    var xhr = new XMLHttpRequest();
    xhr.open('POST', url);
    xhr.responseType = 'json';
    xhr.setRequestHeader('Content-Type', 'application/json');

    xhr.addEventListener('load', function (e) {
      var response = this.response;
      if (this.status >= 400) {
        notifyFailure(this.status + ' ' + this.statusText + ' ' + JSON.stringify(response));
      } else {
        if (response.errorCode) {
          notifyFailure('Error from GA4GH Feature endpoint: ' + JSON.stringify(response));
        } else {
          addFeaturesFromResponse(response);
          o.trigger('newdata', range); // display data as it comes in.
          if (response.nextPageToken) {
            fetchFeaturesForInterval(range, response.nextPageToken, numRequests + 1);
          } else {
            o.trigger('networkdone');
          }
        }
      }
    });
    xhr.addEventListener('error', function (e) {
      notifyFailure('Request failed with status: ' + this.status);
    });

    o.trigger('networkprogress', { numRequests: numRequests });
    xhr.send(JSON.stringify({
      featureSetId: spec.featureSetId,
      pageToken: pageToken,
      pageSize: GENES_PER_REQUEST,
      referenceName: range.contig,
      start: range.start(),
      end: range.stop() }));

  }

  function getFeaturesInRange(range) {
    if (!range) return [];
    return _underscore["default"].filter(genes, function (gene) {return gene.intersects(range);});
  }

  var o = {
    rangeChanged: rangeChanged,
    getFeaturesInRange: getFeaturesInRange,

    // These are here to make Flow happy.
    on: function on() {},
    once: function once() {},
    off: function off() {},
    trigger: function trigger(status, param) {} };

  _underscore["default"].extend(o, _backbone.Events); // Make this an event emitter
  return o;
}

module.exports = {
  create: create };