/**
 * The "glue" between TwoBit.js and GenomeTrack.js.
 *
 * GenomeTrack is pure view code -- it renders data which is already in-memory
 * in the browser.
 *
 * TwoBit is purely for data parsing and fetching. It only knows how to return
 * promises for various genome features.
 *
 * This code acts as a bridge between the two. It maintains a local version of
 * the data, fetching remote data and informing the view when it becomes
 * available.
 *
 * 
 */
'use strict';

var _q = _interopRequireDefault(require("q"));
var _underscore = _interopRequireDefault(require("underscore"));
var _backbone = require("backbone");


var _ContigInterval = _interopRequireDefault(require("../ContigInterval"));
var _TwoBit = _interopRequireDefault(require("../data/TwoBit"));
var _RemoteFile = _interopRequireDefault(require("../RemoteFile"));
var _SequenceStore = _interopRequireDefault(require("../SequenceStore"));
var _utils = _interopRequireDefault(require("../utils"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}function _createForOfIteratorHelper(o, allowArrayLike) {var it;if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {if (it) o = it;var i = 0;var F = function F() {};return { s: F, n: function n() {if (i >= o.length) return { done: true };return { done: false, value: o[i++] };}, e: function e(_e) {throw _e;}, f: F };}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");}var normalCompletion = true,didErr = false,err;return { s: function s() {it = o[Symbol.iterator]();}, n: function n() {var step = it.next();normalCompletion = step.done;return step;}, e: function e(_e2) {didErr = true;err = _e2;}, f: function f() {try {if (!normalCompletion && it["return"] != null) it["return"]();} finally {if (didErr) throw err;}} };}function _unsupportedIterableToArray(o, minLen) {if (!o) return;if (typeof o === "string") return _arrayLikeToArray(o, minLen);var n = Object.prototype.toString.call(o).slice(8, -1);if (n === "Object" && o.constructor) n = o.constructor.name;if (n === "Map" || n === "Set") return Array.from(o);if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);}function _arrayLikeToArray(arr, len) {if (len == null || len > arr.length) len = arr.length;for (var i = 0, arr2 = new Array(len); i < len; i++) {arr2[i] = arr[i];}return arr2;}


// Requests for 2bit ranges are expanded to begin & end at multiples of this
// constant. Doing this means that panning typically won't require
// additional network requests.
var BASE_PAIRS_PER_FETCH = 10000;

var MAX_BASE_PAIRS_TO_FETCH = 100000;
var ZERO_BASED = true;



// Flow type for export.













var createFromTwoBitFile = function createFromTwoBitFile(remoteSource) {
  // Local cache of genomic data.
  var _contigList = [];
  var store = new _SequenceStore["default"]();

  // Ranges for which we have complete information -- no need to hit network.
  var coveredRanges = [];

  function fetch(range) {
    var span = range.length();
    if (span > MAX_BASE_PAIRS_TO_FETCH) {
      // trigger header collection to get information about this chromosome,
      // but don't fetch the actual data.
      return remoteSource._getSequenceHeader(range.contig).then(function (header) {return remoteSource.getContigList();}).then(function (c) {
        _contigList = c;
        o.trigger('contigs', _contigList);
        o.trigger('newdatarefused', range);
      }).done();
    }
    //now we can add region to covered regions
    //doing it earlier would provide inconsistency
    coveredRanges.push(range);
    coveredRanges = _ContigInterval["default"].coalesce(coveredRanges);

    remoteSource.getFeaturesInRange(range.contig, range.start(), range.stop()).
    then(function (letters) {
      if (!letters) return;
      if (letters.length < range.length()) {
        // Probably at EOF
        range = new _ContigInterval["default"](range.contig,
        range.start(),
        range.start() + letters.length - 1);
      }
      store.setRange(range, letters);
    }).then(function () {return remoteSource.getContigList();}).then(function (c) {
      _contigList = c;
      o.trigger('contigs', _contigList);
      o.trigger('newdata', range);
    }).done();
  }

  // This either adds or removes a 'chr' as needed.
  function normalizeRangeSync(range) {

    // check for direct match
    var contigIdx = _underscore["default"].findIndex(_contigList, function (ref) {return range.contig == ref.contig;});

    if (contigIdx >= 0) {
      return range;
    }
    var altContig = _utils["default"].altContigName(range.contig);

    contigIdx = _underscore["default"].findIndex(_contigList, function (ref) {return altContig == ref.contig;});

    if (contigIdx >= 0) {
      return {
        contig: altContig,
        start: range.start,
        stop: range.stop };

    }
    return range; // let it fail with the original contig
  }

  function normalizeRange(range) {
    return contigPromise.then(function () {return normalizeRangeSync(range);});
  }

  // Returns a {"chr12:123" -> "[ATCG]"} mapping for the range.
  function getRange(range) {
    return store.getAsObjects(_ContigInterval["default"].fromGenomeRange(range));
  }

  // Returns a string of base pairs for this range.
  function getRangeAsString(range) {
    if (!range) return '';
    return store.getAsString(_ContigInterval["default"].fromGenomeRange(range));
  }

  // Fetch the contig list immediately.
  var contigPromise = remoteSource.getContigList().then(function (c) {
    _contigList = c;
    o.trigger('contigs', _contigList);
    return c;
  });
  contigPromise.done();

  var o = {
    // The range here is 0-based, inclusive
    rangeChanged: function rangeChanged(newRange) {
      normalizeRange(newRange).then(function (r) {
        var range = new _ContigInterval["default"](r.contig, r.start, r.stop);

        // Check if this interval is already in the cache.
        if (range.isCoveredBy(coveredRanges)) {
          return;
        }

        range = range.round(BASE_PAIRS_PER_FETCH, ZERO_BASED);
        var newRanges = range.complementIntervals(coveredRanges);var _iterator = _createForOfIteratorHelper(

        newRanges),_step;try {for (_iterator.s(); !(_step = _iterator.n()).done;) {var newRange = _step.value;
            fetch(newRange);
          }} catch (err) {_iterator.e(err);} finally {_iterator.f();}
      }).done();
    },
    // The ranges passed to these methods are 0-based
    getRange: getRange,
    getRangeAsString: getRangeAsString,
    contigList: function contigList() {return _contigList;},
    normalizeRange: normalizeRange,

    // These are here to make Flow happy.
    on: function on() {},
    once: function once() {},
    off: function off() {},
    trigger: function trigger(status, param) {} };

  _underscore["default"].extend(o, _backbone.Events); // Make this an event emitter

  return o;
};

function create(data) {
  var url = data.url;
  if (!url) {
    throw new Error("Missing URL from track: ".concat(JSON.stringify(data)));
  }

  return createFromTwoBitFile(new _TwoBit["default"](new _RemoteFile["default"](url)));
}

// Getter/setter for base pairs per fetch.
// This should only be used for testing.
function testBasePairsToFetch(num) {
  if (num) {
    BASE_PAIRS_PER_FETCH = num;
  } else {
    return BASE_PAIRS_PER_FETCH;
  }
}

module.exports = {
  create: create,
  createFromTwoBitFile: createFromTwoBitFile,
  testBasePairsToFetch: testBasePairsToFetch };