
'use strict';

var _chai = require("chai");

var _underscore = _interopRequireDefault(require("underscore"));
var _dataCanvas = _interopRequireDefault(require("data-canvas"));

var _pileup = _interopRequireDefault(require("../main/pileup"));
var _ContigInterval = _interopRequireDefault(require("../main/ContigInterval"));
var _async = require("./async");function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}

describe('pileup', function () {

  var p;
  var testDiv = document.getElementById('testdiv');
  if (!testDiv) throw new Error("Failed to match: testDiv");

  var initialRange = { contig: 'chr17', start: 100, stop: 150 };

  function getTracks() {
    return [
    {
      viz: _pileup["default"].viz.scale(),
      data: _pileup["default"].formats.empty(),
      name: 'Scale' },

    {
      viz: _pileup["default"].viz.location(),
      data: _pileup["default"].formats.empty(),
      name: 'Location' },

    {
      viz: _pileup["default"].viz.genome(),
      isReference: true,
      data: _pileup["default"].formats.twoBit({
        url: '/test-data/test.2bit' }),

      cssClass: 'a' },

    {
      viz: _pileup["default"].viz.variants(),
      data: _pileup["default"].formats.vcf({
        url: '/test-data/snv.chr17.vcf' }),

      cssClass: 'b' },

    {
      viz: _pileup["default"].viz.genes(),
      data: _pileup["default"].formats.bigBed({
        // This file contains just TP53, shifted so that it starts at the
        // beginning of chr17 (to match test.2bit). See test/data/README.md.
        url: '/test-data/tp53.shifted.bb' }),

      cssClass: 'c' },

    {
      viz: _pileup["default"].viz.pileup(),
      data: _pileup["default"].formats.bam({
        url: '/test-data/chr17.1-250.bam',
        indexUrl: '/test-data/chr17.1-250.bam.bai' }),

      cssClass: 'd' }];


  }

  function makePileup() {
    p = _pileup["default"].create(testDiv, {
      range: initialRange,
      tracks: getTracks() });

  }var _dataCanvas$Recording =


  _dataCanvas["default"].RecordingContext,drawnObjects = _dataCanvas$Recording.drawnObjects,drawnObjectsWith = _dataCanvas$Recording.drawnObjectsWith,callsOf = _dataCanvas$Recording.callsOf;

  var uniqDrawnObjectsWith = function uniqDrawnObjectsWith(div, name, f) {
    return _underscore["default"].uniq(
    drawnObjectsWith(div, name, f),
    false, // not sorted
    function (x) {return x.key;});
  };

  // TODO: consider moving this into the data-canvas library
  function hasCanvasAndObjects(div, selector) {
    return div.querySelector(selector + ' canvas') !== null && drawnObjects(div, selector).length > 0;
  }

  var ready = function ready() {return (
      // $FlowIgnore: TODO remove flow suppression
      hasCanvasAndObjects(testDiv, '.reference') &&
      hasCanvasAndObjects(testDiv, '.variants') &&
      hasCanvasAndObjects(testDiv, '.genes') &&
      hasCanvasAndObjects(testDiv, '.pileup'));};


  var rangeChanged = function rangeChanged(range) {return (
      // $FlowIgnore: TODO remove flow suppression
      initialRange != p.getRange());};



  beforeEach(function () {
    _dataCanvas["default"].RecordingContext.recordAll(); // record all data canvases
    testDiv.style.width = '800px';
  });

  afterEach(function () {
    _dataCanvas["default"].RecordingContext.reset();
    if (p) p.destroy();
    testDiv.innerHTML = '';
    testDiv.style.width = '';
  });

  it('should render reference genome and genes', function (done) {
    this.timeout(5000);

    makePileup();

    (0, _async.waitFor)(ready, 5000).
    then(function () {
      var basepairs = drawnObjectsWith(testDiv, '.reference', function (x) {return x.letter;});
      (0, _chai.expect)(basepairs).to.have.length.at.least(10);

      var variants = drawnObjectsWith(testDiv, '.variants', function (x) {return x.alt;});
      (0, _chai.expect)(variants).to.have.length(1);
      (0, _chai.expect)(variants[0].position).to.equal(125);
      (0, _chai.expect)(variants[0].ref).to.equal('G');
      (0, _chai.expect)(variants[0].alt).to.equal('T');

      var geneTexts = callsOf(testDiv, '.genes', 'fillText');
      (0, _chai.expect)(geneTexts).to.have.length(1);
      (0, _chai.expect)(geneTexts[0][1]).to.equal('TP53');

      // Note: there are 11 exons, but two are split into coding/non-coding
      (0, _chai.expect)(callsOf(testDiv, '.genes', 'fillRect')).to.have.length(13);

      // check for reference
      var selectedClass = testDiv.querySelector('div > .a');
      (0, _chai.expect)(selectedClass).to.not.be["null"];
      if (selectedClass != null) {
        (0, _chai.expect)(selectedClass.className).to.equal('track reference a');
      }

      // check for variants
      selectedClass = testDiv.querySelector('div > .b');
      (0, _chai.expect)(selectedClass).to.not.be["null"];
      if (selectedClass != null) {
        (0, _chai.expect)(selectedClass.className).to.equal('track variants b');
      }

      // check for genes
      selectedClass = testDiv.querySelector('div > .c');
      (0, _chai.expect)(selectedClass).to.not.be["null"];
      if (selectedClass != null) {
        (0, _chai.expect)(selectedClass.className).to.equal('track genes c');
      }

      // check for pileup
      selectedClass = testDiv.querySelector('div > .d');
      (0, _chai.expect)(selectedClass).to.not.be["null"];
      if (selectedClass != null) {
        (0, _chai.expect)(selectedClass.className).to.equal('track pileup d');
      }

      (0, _chai.expect)(p.getRange()).to.deep.equal({
        contig: 'chr17',
        start: 100,
        stop: 150 });


      // Four read groups are visible.
      // Due to tiling, some rendered reads may be off-screen.
      var range = p.getRange();
      var cRange = new _ContigInterval["default"](range.contig, range.start, range.stop);
      var visibleReads = uniqDrawnObjectsWith(testDiv, '.pileup', function (x) {return x.span;}).
      filter(function (x) {return x.span.intersects(cRange);});
      (0, _chai.expect)(visibleReads).to.have.length(4);
      done();
    });
  });

  it('should restrict viewing region outside of chromosome', function (done) {
    this.timeout(5000);

    makePileup();

    (0, _async.waitFor)(ready, 5000).
    then(function () {

      (0, _chai.expect)(p.getRange()).to.deep.equal({
        contig: 'chr17',
        start: 100,
        stop: 150 });


      // out of bounds for chromosome
      var range = {
        contig: 'chr17',
        start: 100000000,
        stop: 100000200 };


      p.setRange(range);

      (0, _async.waitFor)(rangeChanged, 5000).
      then(function () {
        (0, _chai.expect)(p.getRange()).to.deep.equal({
          contig: 'chr17',
          start: 0,
          stop: 19198 });

        done();
      });
    });
  });

  it('should save SVG', function (done) {
    this.timeout(5000);

    makePileup();

    (0, _async.waitFor)(ready, 5000).
    then(function () {

      // test zoom in, out and svg
      p.zoomIn();
      (0, _async.waitFor)(rangeChanged, 5000).
      then(function () {
        // not waiting
        (0, _chai.expect)(p.getRange()).to.deep.equal({
          contig: 'chr17',
          start: 112,
          stop: 138 });


        // test conversion to SVG
        p.toSVG().then(function (svg) {
          (0, _chai.expect)(svg).to.contain("svg");
          (0, _chai.expect)(svg).to.contain("chr17");
          (0, _chai.expect)(svg).to.contain("Location");
          (0, _chai.expect)(svg).to.contain("Scale");
          done();
        });
      });
    });
  });
});