/**
 * This tests whether feature information is being shown/drawn correctly
 * in the track.
 *
 * 
 */
'use strict';

var _chai = require("chai");

var _underscore = _interopRequireDefault(require("underscore"));
var _RemoteFile = _interopRequireDefault(require("../../main/RemoteFile"));
var _TwoBit = _interopRequireDefault(require("../../main/data/TwoBit"));
var _TwoBitDataSource = _interopRequireDefault(require("../../main/sources/TwoBitDataSource"));
var _MappedRemoteFile = _interopRequireDefault(require("../MappedRemoteFile"));
var _FakeTwoBit = require("../FakeTwoBit");
var _pileup = _interopRequireDefault(require("../../main/pileup"));
var _dataCanvas = _interopRequireDefault(require("data-canvas"));
var _async = require("../async");

var _pileuputils = require("../../main/viz/pileuputils");

var _testUtils = _interopRequireDefault(require("react-dom/test-utils"));function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}


describe('FeatureTrack', function () {
  var testDiv = document.getElementById('testdiv');
  if (!testDiv) throw new Error("Failed to match: testdiv");

  var drawnObjects = _dataCanvas["default"].RecordingContext.drawnObjects;

  function ready() {
    return testDiv.querySelector('.features canvas') !== null &&
    testDiv.querySelector('.features canvas') !== undefined &&
    drawnObjects(testDiv, '.features').length > 0;
  }

  // Test data files
  var twoBitFile = new _MappedRemoteFile["default"](
  '/test-data/hg19.2bit.mapped',
  [[0, 16383], [691179834, 691183928], [694008946, 694011447]]);

  describe('jsonFeatures', function () {

    beforeEach(function () {
      testDiv.style.width = '800px';
      _dataCanvas["default"].RecordingContext.recordAll();
    });

    afterEach(function () {
      _dataCanvas["default"].RecordingContext.reset();
      // avoid pollution between tests.
      testDiv.innerHTML = '';
    });

    var reference = '';
    var json;

    before(function () {
      var twoBit = new _TwoBit["default"](twoBitFile);
      return twoBit.getFeaturesInRange('chr17', 7500000, 7510000).then(function (seq) {
        reference = seq;
        return new _RemoteFile["default"]('/test-data/features.ga4gh.chr1.120000-125000.chr17.7500000-7515100.json').getAllString();
      }).then(function (data) {
        (0, _chai.expect)(data).to.have.length.above(0);
        json = data;
      });
    });

    it('should render features with json', function () {
      var featureClickedData = null;
      var featureClicked = function featureClicked(data) {
        featureClickedData = data;
      };

      var fakeTwoBit = new _FakeTwoBit.FakeTwoBit(twoBitFile),
      referenceSource = _TwoBitDataSource["default"].createFromTwoBitFile(fakeTwoBit);

      // Release the reference first.
      fakeTwoBit.release(reference);

      var p = _pileup["default"].create(testDiv, {
        range: { contig: 'chr1', start: 130000, stop: 135000 },
        tracks: [
        {
          viz: _pileup["default"].viz.genome(),
          data: referenceSource,
          isReference: true },

        {
          viz: _pileup["default"].viz.features(),
          data: _pileup["default"].formats.featureJson(json),
          options: { onFeatureClicked: featureClicked } }] });




      return (0, _async.waitFor)(ready, 2000).
      then(function () {
        var features = drawnObjects(testDiv, '.features');
        // there can be duplicates in the case where features are
        // overlapping  more than one section of the canvas
        features = _underscore["default"].uniq(features, false, function (x) {
          return x.position.start();
        });

        (0, _chai.expect)(features).to.have.length(4);
        (0, _chai.expect)(features.map(function (f) {return f.position.start();})).to.deep.equal(
        [89295, 92230, 110953, 120725]);

        var height = Math.round((0, _pileuputils.yForRow)(4) * window.devicePixelRatio); // should be 4 rows
        features = testDiv.querySelector('.features');
        (0, _chai.expect)(features).to.not.be["null"];
        if (features != null) {
          (0, _chai.expect)(features.style.height).to.equal("".concat(height, "px"));
        }

        // check clicking on feature
        var canvasList = testDiv.getElementsByTagName('canvas');
        var canvas = canvasList[1];
        (0, _chai.expect)(featureClickedData).to.be["null"];
        _testUtils["default"].Simulate.click(canvas, { nativeEvent: { offsetX: 430, offsetY: 50 * window.devicePixelRatio } });
        (0, _chai.expect)(featureClickedData).to.not.be["null"];

        p.destroy();
      });
    });
  });

  describe('bigBedFeatures', function () {

    beforeEach(function () {
      testDiv.style.width = '800px';
      _dataCanvas["default"].RecordingContext.recordAll();
    });

    afterEach(function () {
      _dataCanvas["default"].RecordingContext.reset();
      // avoid pollution between tests.
      testDiv.innerHTML = '';
    });

    var reference = '';

    before(function () {
      var twoBit = new _TwoBit["default"](twoBitFile);
      return twoBit.getFeaturesInRange('chr17', 7500000, 7510000).then(function (seq) {
        reference = seq;
      });
    });

    it('should render features with bigBed file', function () {
      var fakeTwoBit = new _FakeTwoBit.FakeTwoBit(twoBitFile),
      referenceSource = _TwoBitDataSource["default"].createFromTwoBitFile(fakeTwoBit);

      // Release the reference first.
      fakeTwoBit.release(reference);

      var p = _pileup["default"].create(testDiv, {
        range: { contig: 'chr17', start: 10000, stop: 16500 },
        tracks: [
        {
          viz: _pileup["default"].viz.genome(),
          data: referenceSource,
          isReference: true },

        {
          viz: _pileup["default"].viz.features(),
          data: _pileup["default"].formats.bigBed({
            url: '/test-data/chr17.22.10000-21000.bb' }),

          name: 'Features' }] });





      return (0, _async.waitFor)(ready, 2000).then(function () {
        var features = drawnObjects(testDiv, '.features');
        // there can be duplicates in the case where features are
        // overlapping  more than one section of the canvas
        features = _underscore["default"].uniq(features, false, function (x) {
          return x.position.start();
        });

        (0, _chai.expect)(features).to.have.length.at.least(2);

        p.setRange({ contig: 'chr22', start: 20000, stop: 21000 });
      }).delay(300).then(function () {
        var features = drawnObjects(testDiv, '.features');
        // there can be duplicates in the case where features are
        // overlapping  more than one section of the canvas
        features = _underscore["default"].uniq(features, false, function (x) {
          return x.position.start();
        });

        (0, _chai.expect)(features).to.have.length(10);

        // canvas height should be maxed out, should not exceed parent height limits
        var expectedHeight = Math.round(150 * window.devicePixelRatio);
        var featureCanvas = testDiv.querySelector('.features');
        (0, _chai.expect)(featureCanvas).to.not.be["null"];
        if (featureCanvas != null) {
          (0, _chai.expect)(featureCanvas.style.height).to.equal("".concat(expectedHeight, "px"));
        }
        p.destroy();
      });
    });
  });
});