/**
 * This tests that the Controls and reference track render correctly, even when
 * an externally-set range uses a different chromosome naming system (e.g. '17'
 * vs 'chr17'). See https://github.com/hammerlab/pileup.js/issues/146
 * 
 */

'use strict';

var _chai = require("chai");

var _testUtils = _interopRequireDefault(require("react-dom/test-utils"));

var _pileup = _interopRequireDefault(require("../../main/pileup"));
var _TwoBit = _interopRequireDefault(require("../../main/data/TwoBit"));
var _TwoBitDataSource = _interopRequireDefault(require("../../main/sources/TwoBitDataSource"));
var _dataCanvas = _interopRequireDefault(require("data-canvas"));
var _MappedRemoteFile = _interopRequireDefault(require("../MappedRemoteFile"));
var _async = require("../async");function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}function _slicedToArray(arr, i) {return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();}function _nonIterableRest() {throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");}function _unsupportedIterableToArray(o, minLen) {if (!o) return;if (typeof o === "string") return _arrayLikeToArray(o, minLen);var n = Object.prototype.toString.call(o).slice(8, -1);if (n === "Object" && o.constructor) n = o.constructor.name;if (n === "Map" || n === "Set") return Array.from(o);if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);}function _arrayLikeToArray(arr, len) {if (len == null || len > arr.length) len = arr.length;for (var i = 0, arr2 = new Array(len); i < len; i++) {arr2[i] = arr[i];}return arr2;}function _iterableToArrayLimit(arr, i) {if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return;var _arr = [];var _n = true;var _d = false;var _e = undefined;try {for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {_arr.push(_s.value);if (i && _arr.length === i) break;}} catch (err) {_d = true;_e = err;} finally {try {if (!_n && _i["return"] != null) _i["return"]();} finally {if (_d) throw _e;}}return _arr;}function _arrayWithHoles(arr) {if (Array.isArray(arr)) return arr;}

describe('GenomeTrack', function () {
  var testDiv = document.getElementById('testdiv');
  if (!testDiv) throw new Error("Failed to match: testdiv");

  beforeEach(function () {
    // A fixed width container results in predictable x-positions for mismatches.
    testDiv.style.width = '800px';
    _dataCanvas["default"].RecordingContext.recordAll();
  });

  afterEach(function () {
    _dataCanvas["default"].RecordingContext.reset();
    // avoid pollution between tests.
    testDiv.innerHTML = '';
    testDiv.style.width = '';
  });

  var twoBitFile = new _MappedRemoteFile["default"](
  '/test-data/hg19.2bit.mapped',
  [[0, 16383], [691179834, 691183928], [694008946, 694011447]]),
  referenceSource = _TwoBitDataSource["default"].createFromTwoBitFile(new _TwoBit["default"](twoBitFile));var

  drawnObjects = _dataCanvas["default"].RecordingContext.drawnObjects;
  var hasReference = function hasReference() {
    // The reference initially shows "unknown" base pairs, so we have to
    // check for a specific known one to ensure that it's really loaded.
    return testDiv.querySelector('canvas') != null &&
    drawnObjects(testDiv, '.reference').length > 0;
  };

  var referenceTrackLoaded = function referenceTrackLoaded() {
    //this can be done in a preatier way
    return testDiv.querySelector('canvas') !== null;
  };

  it('should tolerate non-chr ranges', function () {
    var p = _pileup["default"].create(testDiv, {
      range: { contig: '17', start: 7500730, stop: 7500790 },
      tracks: [
      {
        data: referenceSource,
        viz: _pileup["default"].viz.genome(),
        isReference: true }] });




    return (0, _async.waitFor)(hasReference, 2000).then(function () {
      // The contig selector should list have "chr" prefixes & an active selection.
      var options = testDiv.querySelectorAll('option');
      (0, _chai.expect)(options).to.have.length.above(20);
      (0, _chai.expect)(options[0].textContent).to.equal('chr1');
      var opt17 = options[17];
      (0, _chai.expect)(opt17.textContent).to.equal('chr17');
      (0, _chai.expect)(opt17.selected).to.be["true"];
      (0, _chai.expect)(p.getRange()).to.deep.equal({
        contig: 'chr17',
        start: 7500730,
        stop: 7500790 });

      p.destroy();
    });
  });

  var getInputs = function getInputs(selector) {
    var els = testDiv.querySelectorAll(selector);
    // note: this isn't really true, but it makes flow happy
    return els;
  };

  /**
   * Test case show situation when we zoom in from very global view
   * (range span is milions of nucleotides) into very narrow view
   * (tens of nucleotides).
   */
  it('should zoom from huge zoom out', function () {

    var p = _pileup["default"].create(testDiv, {
      range: { contig: '17', start: 0, stop: 114529884 },
      tracks: [{
        data: referenceSource,
        viz: _pileup["default"].viz.genome(),
        isReference: true }] });




    (0, _chai.expect)(testDiv.querySelectorAll('.zoom-controls')).to.have.length(1);

    var buttons = testDiv.querySelectorAll('.controls button');var _buttons = _slicedToArray(
    buttons, 3),goBtn = _buttons[0],minusBtn = _buttons[1],plusBtn = _buttons[2];var _getInputs =
    getInputs('.controls input[type="text"]'),_getInputs2 = _slicedToArray(_getInputs, 1),locationTxt = _getInputs2[0];
    (0, _chai.expect)(goBtn.textContent).to.equal('Go');
    (0, _chai.expect)(minusBtn.className).to.equal('btn-zoom-out');
    (0, _chai.expect)(plusBtn.className).to.equal('btn-zoom-in');

    return (0, _async.waitFor)(referenceTrackLoaded, 2000).then(function () {
      //in global view we shouldn't see reference track
      (0, _chai.expect)(hasReference()).to.be["false"];
      p.setRange({ contig: '17', start: 7500725, stop: 7500775 });
    }).delay(300).then(function () {
      //after zoom in we should see reference track
      (0, _chai.expect)(hasReference()).to.be["true"];
      (0, _chai.expect)(locationTxt.value).to.equal('7,500,725-7,500,775');
      p.destroy();
    });
  });

  it('should zoom in and out', function () {
    var p = _pileup["default"].create(testDiv, {
      range: { contig: '17', start: 7500725, stop: 7500775 },
      tracks: [
      {
        data: referenceSource,
        viz: _pileup["default"].viz.genome(),
        isReference: true }] });




    (0, _chai.expect)(testDiv.querySelectorAll('.zoom-controls')).to.have.length(1);

    var buttons = testDiv.querySelectorAll('.controls button');var _buttons2 = _slicedToArray(
    buttons, 3),goBtn = _buttons2[0],minusBtn = _buttons2[1],plusBtn = _buttons2[2];var _getInputs3 =
    getInputs('.controls input[type="text"]'),_getInputs4 = _slicedToArray(_getInputs3, 1),locationTxt = _getInputs4[0];
    (0, _chai.expect)(goBtn.textContent).to.equal('Go');
    (0, _chai.expect)(minusBtn.className).to.equal('btn-zoom-out');
    (0, _chai.expect)(plusBtn.className).to.equal('btn-zoom-in');

    return (0, _async.waitFor)(hasReference, 2000).then(function () {
      (0, _chai.expect)(locationTxt.value).to.equal('7,500,725-7,500,775');
      _testUtils["default"].Simulate.click(minusBtn);
    }).delay(50).then(function () {
      (0, _chai.expect)(p.getRange()).to.deep.equal({
        contig: 'chr17',
        start: 7500700,
        stop: 7500800 });

      (0, _chai.expect)(locationTxt.value).to.equal('7,500,700-7,500,800');
      _testUtils["default"].Simulate.click(plusBtn);
    }).delay(50).then(function () {
      (0, _chai.expect)(p.getRange()).to.deep.equal({
        contig: 'chr17',
        start: 7500725,
        stop: 7500775 });

      (0, _chai.expect)(locationTxt.value).to.equal('7,500,725-7,500,775');
      p.destroy();
    });
  });

  it('should zoom according to the value of the slider', function () {
    var p = _pileup["default"].create(testDiv, {
      range: { contig: '17', start: 7500725, stop: 7500775 },
      tracks: [
      {
        data: referenceSource,
        viz: _pileup["default"].viz.genome(),
        isReference: true }] });




    (0, _chai.expect)(testDiv.querySelectorAll('.zoom-controls')).to.have.length(1);
    (0, _chai.expect)(testDiv.querySelectorAll('.zoom-slider')).to.have.length(1);
    // querySelectorAll returns HTMLElement
    // cast to any and then to HTMLInputElement to make flow happy
    var slider = testDiv.querySelectorAll('.zoom-slider')[0];var _getInputs5 =
    getInputs('.controls input[type="text"]'),_getInputs6 = _slicedToArray(_getInputs5, 1),locationTxt = _getInputs6[0];

    return (0, _async.waitFor)(hasReference, 2000).then(function () {
      slider.value = "-1";
      _testUtils["default"].Simulate.input(slider);

    }).delay(50).then(function () {

      (0, _chai.expect)(p.getRange()).to.deep.equal({
        contig: 'chr17',
        start: 7500748,
        stop: 7500752 });

      (0, _chai.expect)(locationTxt.value).to.equal('7,500,748-7,500,752');
      slider.value = "-2";
      _testUtils["default"].Simulate.input(slider);
    }).delay(50).then(function () {
      (0, _chai.expect)(p.getRange()).to.deep.equal({
        contig: 'chr17',
        start: 7500746,
        stop: 7500754 });

      (0, _chai.expect)(locationTxt.value).to.equal('7,500,746-7,500,754');
      slider.value = "-5";
      _testUtils["default"].Simulate.input(slider);
    }).delay(50).then(function () {
      (0, _chai.expect)(p.getRange()).to.deep.equal({
        contig: 'chr17',
        start: 7500718,
        stop: 7500782 });

      (0, _chai.expect)(locationTxt.value).to.equal('7,500,718-7,500,782');
      p.destroy();
    });
  });

  it('should accept user-entered locations', function () {
    var p = _pileup["default"].create(testDiv, {
      range: { contig: '17', start: 7500725, stop: 7500775 },
      tracks: [
      {
        data: referenceSource,
        viz: _pileup["default"].viz.genome(),
        isReference: true }] });var _getInputs7 =




    getInputs('.controls input[type="text"]'),_getInputs8 = _slicedToArray(_getInputs7, 1),locationTxt = _getInputs8[0];var _testDiv$querySelecto =
    testDiv.querySelectorAll('.controls button'),_testDiv$querySelecto2 = _slicedToArray(_testDiv$querySelecto, 1),goBtn = _testDiv$querySelecto2[0];
    (0, _chai.expect)(goBtn.textContent).to.equal('Go');

    return (0, _async.waitFor)(hasReference, 2000).then(function () {
      (0, _chai.expect)(locationTxt.value).to.equal('7,500,725-7,500,775');
      locationTxt.value = '17:7500745-7500785';
      _testUtils["default"].Simulate.click(goBtn);
    }).delay(50).then(function () {
      (0, _chai.expect)(p.getRange()).to.deep.equal({
        contig: 'chr17', // note: not '17'
        start: 7500745,
        stop: 7500785 });

      (0, _chai.expect)(locationTxt.value).to.equal('7,500,745-7,500,785');
      p.destroy();
    });
  });
});