/**
 * This tests that the Controls and reference track render correctly, even when
 * an externally-set range uses a different chromosome naming system (e.g. '17'
 * vs 'chr17'). See https://github.com/hammerlab/pileup.js/issues/146
 * 
 */

'use strict';

var _chai = require("chai");

var _sinon = _interopRequireDefault(require("sinon"));

var _pileup = _interopRequireDefault(require("../../main/pileup"));
var _dataCanvas = _interopRequireDefault(require("data-canvas"));
var _async = require("../async");
var _RemoteFile = _interopRequireDefault(require("../../main/RemoteFile"));
var _TwoBit = _interopRequireDefault(require("../../main/data/TwoBit"));
var _TwoBitDataSource = _interopRequireDefault(require("../../main/sources/TwoBitDataSource"));
var _MappedRemoteFile = _interopRequireDefault(require("../MappedRemoteFile"));
var _FakeTwoBit = require("../FakeTwoBit");function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { "default": obj };}

describe('GenotypeTrack', function () {
  var server = null,response;
  var reference = '';
  var fakeTwoBit, referenceSource;

  var testDiv = document.getElementById('testdiv');
  if (!testDiv) throw new Error("Failed to match: testdiv");

  // Test data files
  var twoBitFile = new _MappedRemoteFile["default"](
  '/test-data/hg19.2bit.mapped',
  [[0, 16383], [691179834, 691183928], [694008946, 694011447]]);


  before(function () {
    var twoBit = new _TwoBit["default"](twoBitFile);
    return twoBit.getFeaturesInRange('17', 7500000, 7510000).then(function (seq) {
      reference = seq;
      return new _RemoteFile["default"]('/test-data/variants.ga4gh.chr1.10000-11000.json').getAllString();
    }).then(function (data) {
      response = data;

      server = _sinon["default"].createFakeServer();
      server.autoRespond = true;

      // Sinon should ignore 2bit request. RemoteFile handles this request.
      _sinon["default"].fakeServer.xhr.useFilters = true;
      _sinon["default"].fakeServer.xhr.addFilter(function (method, url) {
        return url === '/test-data/test.2bit';
      });
      // Sinon should ignore vcf file requests. RemoteFile handles this request.
      _sinon["default"].fakeServer.xhr.addFilter(function (method, url) {
        return url === '/test-data/test.vcf';
      });

      fakeTwoBit = new _FakeTwoBit.FakeTwoBit(twoBitFile),
      referenceSource = _TwoBitDataSource["default"].createFromTwoBitFile(fakeTwoBit);

      // Release the reference first.
      fakeTwoBit.release(reference);

    });

  });

  after(function () {
    server.restore();
  });

  beforeEach(function () {
    testDiv.style.width = '800px';
    _dataCanvas["default"].RecordingContext.recordAll();
  });

  afterEach(function () {
    _dataCanvas["default"].RecordingContext.reset();
    // avoid pollution between tests.
    testDiv.innerHTML = '';
  });

  var drawnObjects = _dataCanvas["default"].RecordingContext.drawnObjects;

  function ready() {
    return testDiv.getElementsByTagName('canvas').length > 0 &&
    drawnObjects(testDiv, '.genotypeRows').length > 1;
  }

  it('should render genotypes from vcf file', function () {

    var p = _pileup["default"].create(testDiv, {
      range: { contig: '17', start: 9386380, stop: 9386390 },
      tracks: [
      {
        viz: _pileup["default"].viz.genome(),
        data: referenceSource,
        isReference: true },

      {
        data: _pileup["default"].formats.vcf({
          url: '/test-data/test.vcf' }),

        viz: _pileup["default"].viz.genotypes() }] });




    return (0, _async.waitFor)(ready, 2000).
    then(function () {
      var labels = drawnObjects(testDiv, '.genotypeLabels');

      (0, _chai.expect)(labels).to.have.length(2);
      (0, _chai.expect)(labels).to.deep.equal(
      ["NORMAL", "TUMOR"]);

      var genotypes = drawnObjects(testDiv, '.genotypeRows');

      (0, _chai.expect)(genotypes).to.have.length(3);
      (0, _chai.expect)(genotypes[0]).to.deep.equal("NORMAL");
      (0, _chai.expect)(genotypes[1]).to.deep.equal("TUMOR");
      (0, _chai.expect)(genotypes[2].position).to.deep.equal(9386385);


      p.destroy();
    });
  });

  it('should render genotypes from GA4GH Variants', function () {


    server.respondWith('POST', '/v0.6.0/variants/search',
    [200, { "Content-Type": "application/json" }, response]);


    var callSetId = "WyIxa2dlbm9tZXMiLCJ2cyIsInBoYXNlMy1yZWxlYXNlIiwiSEcwMDA5NiJd";
    var callSetName = "HG00096";

    var p = _pileup["default"].create(testDiv, {
      range: { contig: '17', start: 9386380, stop: 9386390 },
      tracks: [
      {
        viz: _pileup["default"].viz.genome(),
        data: referenceSource,
        isReference: true },

      {
        data: _pileup["default"].formats.GAVariant({
          endpoint: '/v0.6.0',
          variantSetId: "WyIxa2dlbm9tZXMiLCJ2cyIsInBoYXNlMy1yZWxlYXNlIl0",
          callSetIds: [callSetId],
          callSetNames: [callSetName] }),

        viz: _pileup["default"].viz.genotypes() }] });




    return (0, _async.waitFor)(ready, 2000).
    then(function () {
      var labels = drawnObjects(testDiv, '.genotypeLabels');

      (0, _chai.expect)(labels).to.have.length(1);
      (0, _chai.expect)(labels).to.deep.equal(
      [callSetName]);

      var genotypes = drawnObjects(testDiv, '.genotypeRows');

      (0, _chai.expect)(genotypes).to.have.length(2);
      (0, _chai.expect)(genotypes[0]).to.deep.equal(callSetName);
      (0, _chai.expect)(genotypes[1].position).to.deep.equal(9386385);

      p.destroy();
    });

  });

  it('should render genotypes from GA4GH Variants when call names are not specified', function () {


    server.respondWith('POST', '/v0.6.0/variants/search',
    [200, { "Content-Type": "application/json" }, response]);


    var callSetId = "WyIxa2dlbm9tZXMiLCJ2cyIsInBoYXNlMy1yZWxlYXNlIiwiSEcwMDA5NiJd";

    var p = _pileup["default"].create(testDiv, {
      range: { contig: '17', start: 9386380, stop: 9386390 },
      tracks: [
      {
        viz: _pileup["default"].viz.genome(),
        data: referenceSource,
        isReference: true },

      {
        data: _pileup["default"].formats.GAVariant({
          endpoint: '/v0.6.0',
          variantSetId: "WyIxa2dlbm9tZXMiLCJ2cyIsInBoYXNlMy1yZWxlYXNlIl0",
          callSetIds: [callSetId] }),

        viz: _pileup["default"].viz.genotypes() }] });





    return (0, _async.waitFor)(ready, 2000).
    then(function () {
      var labels = drawnObjects(testDiv, '.genotypeLabels');

      (0, _chai.expect)(labels).to.have.length(1);
      (0, _chai.expect)(labels).to.deep.equal(
      [callSetId]);

      var genotypes = drawnObjects(testDiv, '.genotypeRows');

      (0, _chai.expect)(genotypes).to.have.length(2);
      (0, _chai.expect)(genotypes[0]).to.deep.equal(callSetId);
      (0, _chai.expect)(genotypes[1].position).to.deep.equal(9386385);

      p.destroy();
    });

  });

  it('should render genotypes from GA4GH Variant JSON', function () {

    var p = _pileup["default"].create(testDiv, {
      range: { contig: '17', start: 9386380, stop: 9386390 },
      tracks: [
      {
        viz: _pileup["default"].viz.genome(),
        data: referenceSource,
        isReference: true },

      {
        data: _pileup["default"].formats.variantJson(response),
        viz: _pileup["default"].viz.genotypes() }] });




    var callSetName = "HG00096";

    return (0, _async.waitFor)(ready, 2000).
    then(function () {
      var labels = drawnObjects(testDiv, '.genotypeLabels');

      (0, _chai.expect)(labels).to.have.length(1);
      (0, _chai.expect)(labels).to.deep.equal(
      [callSetName]);

      var genotypes = drawnObjects(testDiv, '.genotypeRows');

      (0, _chai.expect)(genotypes).to.have.length(2);
      (0, _chai.expect)(genotypes[0]).to.deep.equal(callSetName);
      (0, _chai.expect)(genotypes[1].position).to.deep.equal(9386385);

      p.destroy();
    });
  });

});